/* src/config_coinutils.h.  Generated by configure.  */
/* inc/config_coinutils.h.in.  */

#ifndef __CONFIG_COINUTILS_H__
#define __CONFIG_COINUTILS_H__

/* Define to 1 if stdint.h is available for CoinUtils */
/* #undef COINUTILS_HAS_STDINT_H */

/* Define to 1 if stdint.h is available for CoinUtils */
#define COINUTILS_HAS_CSTDINT 1

/* Version number of project */
#define COINUTILS_VERSION "2.11.6"

/* Major Version number of project */
#define COINUTILS_VERSION_MAJOR 2

/* Minor Version number of project */
#define COINUTILS_VERSION_MINOR 11

/* Release Version number of project */
#define COINUTILS_VERSION_RELEASE 6

/* Define to 64bit integer type */
#define COIN_INT64_T int64_t

/* Define to integer type capturing pointer */
#define COIN_INTPTR_T intptr_t

/* Define to 64bit unsigned integer type */
#define COIN_UINT64_T int64_t

#endif
